@@ -1,6 +1,7 @@
 import os, re, time, urllib
 from django.utils.translation import ugettext as _
 from exceptions import ElfinderErrorMessages, VolumeNotFoundError, DirNotFoundError, FileNotFoundError, NamedError, NotAnImageError
+from utils.volumes import instantiate_driver
 
 class ElfinderConnector:
     """
@@ -39,6 +40,10 @@ class ElfinderConnector:
     }
 
     def __init__(self, opts, session = None):
+
+        if not 'roots' in opts:
+            opts['roots'] = []
+
         self._volumes = {}
         self._default = None
         self._loaded = False
@@ -49,32 +54,21 @@ class ElfinderConnector:
         self._mountErrors = []
         
         #TODO: Use signals instead of the original connector's binding mechanism
-        if not 'roots' in opts:
-            opts['roots'] = []
-        
+
         #for root in self.getNetVolumes():
         #    opts['roots'].append(root)
 
         for o in opts['roots']:
-            class_ = o['driver'] if 'driver' in o else ''  
-                      
-            try:
-                volume = class_()
-            except TypeError:
-                self._mountErrors.append('Driver "%s" does not exist' % class_)
-            except:
-                #TODO: should we fail silently?
-                raise
 
             try:
-                volume.mount(o)
+                volume = instantiate_driver(o)
             except Exception as e:
-                self._mountErrors.append('Driver "%s" " %s' % (class_, e))
-                break
-            
+                self._mountErrors.append(e.__unicode__())
+                continue
+
             id_ = volume.id()
             self._volumes[id_] = volume
-            if not self._default and volume.isReadable():
+            if not self._default and volume.is_readable():
                 self._default = self._volumes[id_]
 
         self._loaded = (self._default is not None)
@@ -95,7 +89,7 @@ class ElfinderConnector:
         """
         Check if command exists
         """
-        return cmd in self._commands and hasattr(self, cmd) and callable(getattr(self, cmd))
+        return cmd in self._commands and hasattr(self, '_%s' % cmd) and callable(getattr(self, '_%s' % cmd))
     
     def commandArgsList(self, cmd):
         """
@@ -103,13 +97,6 @@ class ElfinderConnector:
         """
         return self._commands[cmd] if self.commandExists(cmd) else {}
     
-    def realpath(self, hash_):
-        """
-        Return file real path.
-        Raises a VolumeNotFoundError exception if volume was not found.
-        """
-        return self._volume(hash_).realpath(hash_)
-    
     #def getNetVolumes(self):
     #    """
     #    Return  network volumes config.
@@ -143,81 +130,86 @@ class ElfinderConnector:
         """        
         if not self._loaded:
             return { 'error' : self.error(ElfinderErrorMessages.ERROR_CONF, ElfinderErrorMessages.ERROR_CONF_NO_VOL)}
-
-        if 'mimes' in kwargs:
-            for id_ in self._volumes:
-                self._volumes[id_].setMimesFilter(kwargs['mimes'])
         
         if not self.commandExists(cmd):
             return { 'error' : self.error(ElfinderErrorMessages.ERROR_UNKNOWN_CMD, cmd)}
         
+        #check all required arguments are provided
         for arg, req in self.commandArgsList(cmd).items():
-            if req and not kwargs[arg]:
+            if req and (not arg in kwargs or not kwargs[arg]):
                 return {'error' : self.error(ElfinderErrorMessages.ERROR_INV_PARAMS, cmd)}
         
-        result = getattr(self, cmd)(**kwargs)
+        #set mimes filter and pop mimes from the arguments list
+        if 'mimes' in kwargs:
+            for id_ in self._volumes:
+                self._volumes[id_].set_mimes_filter(kwargs['mimes'])
+            kwargs.pop('mimes')
+
+        debug = self._debug or ('debug' in kwargs and int(kwargs['debug']))
+        #remove debug kewyord argument  
+        if 'debug' in kwargs:
+            kwargs.pop('debug')
+
+        result = getattr(self, '_%s' % cmd)(**kwargs)
         
+        #checked for removed items as these are not directly returned
         if 'removed' in result:
             for id_ in self._volumes:
                 result['removed'] += self._volumes[id_].removed()
-                self._volumes[id_].resetRemoved()
+                self._volumes[id_].reset_removed()
+            #replace removed files info with removed files hashes and filter out duplicates
+            result['removed'] = list(set([f['hash'] for f in result['removed']]))
         
         #call handlers for this command
         #TODO: a signal must be sent here
         
-        #replace removed files info with removed files hashes
-        if 'removed' in result:
-            removed = []
-            for file_ in result['removed']:
-                if not file_['hash'] in removed:
-                    removed.append(file_['hash'])
-            result['removed'] = removed
-
-        #remove hidden files and filter files by mimetypes
-        if 'added' in result:
-            result['added'] = self._filter(result['added'])
-        
-        #remove hidden files and filter files by mimetypes
-        if 'changed' in result:
-            result['changed'] = self._filter(result['changed'])
-        
-        if self._debug or ('debug' in kwargs and kwargs['debug']):
+        if debug:
             result['debug'] = {
-                'connector' : 'yawd-python',
+                'connector' : 'yawd-elfinder',
                 'time' : time.time() - self._time,
                 'upload' : self._uploadDebug,
-                'volumes' : [],
+                'volumes' : [v.debug() for v in self._volumes.values()],
                 'mountErrors' : self._mountErrors
             }
-            
-            for id_,volume in self._volumes.items():
-                result['debug']['volumes'].append(volume.debug())
-            
+
         return result
 
-    def open(self, target='', init=False, tree=False, debug=False):
+    def _open(self, target='', init=False, tree=False):
         """
         **Command**: Open a directory
         
         Return:
             An array with following elements:
                 :cwd:          opened directory information
-                :files:        opened directory content [and dirs tree if kwargs['tree'] is ``True``]
-                :api:          api version (if kwargs['init'] is ``True``)
-                :uplMaxSize:   The maximum allowed upload size (if kwargs['init'] is ``True``)
+                :options:      the volume options
+                :files:        opened directory content [and dirs tree if 'tree' argument is ``True``]
+                :api:          api version (if 'init' argument is ``True``)
+                :uplMaxSize:   The maximum allowed upload size (if 'init' argument is ``True``)
                 :error:        on failed
+                
+        This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
-        hash_ = 'default folder' if init else '#%s' % target
+
+        if isinstance(init, basestring):
+            init = int(init)
+            
+        if isinstance(tree, basestring):
+            tree = int(tree)
 
         if not init and not target:
             return {'error' : self.error(ElfinderErrorMessages.ERROR_INV_PARAMS, 'open')}
 
+        #display name for use in error messages
+        display_hash = 'default folder' if init else '#%s' % target
+
         #detect volume
         try:
             volume = self._volume(target)
         except VolumeNotFoundError as e:
             if not init:
-                return {'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, hash_, e)}
+                return {'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, display_hash, e)}
             else:
                 #on init request we can get invalid dir hash -
                 #dir which can not be opened now, but remembered by client,
@@ -225,20 +217,20 @@ class ElfinderConnector:
                 volume = self._default
         
         try:
-            cwd = volume.dir(hash_=target, resolveLink=True)
+            cwd = volume.dir(hash_=target, resolve_link=True)
             if not cwd['read'] and init:
                 try:
-                    cwd = volume.dir(hash_=volume.defaultPath(), resolveLink=True)
+                    cwd = volume.dir(hash_=volume.default_path(), resolve_link=True)
                 except (DirNotFoundError, FileNotFoundError) as e:
-                    return {'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, hash_, e)}
+                    return {'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, display_hash, e)}
         except (DirNotFoundError, FileNotFoundError) as e:
             if init:
-                cwd = volume.dir(hash_=volume.defaultPath(), resolveLink=True)
+                cwd = volume.dir(hash_=volume.default_path(), resolve_link=True)
             else:
-                return {'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, hash_, e)}
+                return {'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, display_hash, e)}
 
         if not cwd['read']:
-            return {'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, hash_, ElfinderErrorMessages.ERROR_PERM_DENIED)}
+            return {'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, display_hash, ElfinderErrorMessages.ERROR_PERM_DENIED)}
 
         files = []
         #get folder trees
@@ -246,7 +238,7 @@ class ElfinderConnector:
             for id_ in self._volumes:
                 files += self._volumes[id_].tree(exclude=target)
         
-        #get current working directory files list and add to files if not exists in it
+        #get current working directory files list and add to files if not already present
         try:
             ls = volume.scandir(cwd['hash'])
         except Exception as e:
@@ -258,47 +250,55 @@ class ElfinderConnector:
 
         result = {
             'cwd' : cwd,
-            'options' : volume.options(target),
+            'options' : volume.options(cwd['hash']),
             'files' : files
         }
 
         if init:
             result['api'] = self._version
             result['netDrivers'] = self._netDrivers.keys()
-            #TODO: Do we need Upload Max Size?
+            result['uplMaxSize'] = volume.upload_max_size()
         
         return result
 
-    def ls(self, target, mimes=[], debug=False):
+    def _ls(self, target):
         """
-        **Command**: Return a directory's file list
+        **Command**: Return a directory's file list. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         try:
             return { 'list' : self._volume(target).ls(target) }
         except:
             return { 'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, '#%s' % target) }
 
-    def tree(self, target, debug=False):
+    def _tree(self, target):
         """
-        **Command**: Return subdirs for required directory
+        **Command**: Return subdirs for required directory. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         try:
             return { 'tree' : self._volume(target).tree(target) }
         except:
             return { 'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, '#%s' % target) }
     
-    def parents(self, target, debug=False):
+    def _parents(self, target):
         """
-        **Command**: Return parents dir for required directory
+        **Command**: Return parents dir for required directory. this method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         try:
             return {'tree' : self._volume(target).parents(target) }
         except:
             return { 'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, u'#%s' % target) }
 
-    def tmb(self, targets, debug=False):
+    def _tmb(self, targets):
         """
-        **Command**: Return new automatically-created thumbnails list
+        **Command**: Return new automatically-created thumbnails list. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         result  = { 'images' : {} }
         for target in targets:
@@ -311,7 +311,7 @@ class ElfinderConnector:
 
         return result
     
-    def file(self, target, request=None, download=False, debug=False):
+    def _file(self, target, request=None, download=False):
         """
         **Command**: Get a file
         
@@ -320,7 +320,15 @@ class ElfinderConnector:
         
         Return:
             An array containing an opened file pointer, the root itself and the required response headers
+            
+        This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
+        
+        if isinstance(download, basestring):
+            download = int(download)
+        
         try:
             volume = self._volume(target)
             file_ = volume.file(target)
@@ -372,9 +380,13 @@ class ElfinderConnector:
 
         return result
 
-    def size(self, targets, debug=False):
+    def _size(self, targets):
         """
         **Command**: Count total file size of all directories in ``targets`` param.
+        
+        This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         size = 0
         
@@ -392,9 +404,11 @@ class ElfinderConnector:
 
         return { 'size' : size }
 
-    def mkdir(self, target, name, debug=False):
+    def _mkdir(self, target, name):
         """
-        **Command**: Create a new directory
+        **Command**: Create a new directory. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         try:
             volume = self._volume(target)
@@ -404,13 +418,15 @@ class ElfinderConnector:
         try:
             return { 'added' : [volume.mkdir(target, name)] }
         except NamedError as e:
-            return { 'error' : self.error(ElfinderErrorMessages.ERROR_MKDIR, name, e, e.name) }
+            return { 'error' : self.error(e, e.name, ElfinderErrorMessages.ERROR_MKDIR) }
         except Exception as e:
             return { 'error': self.error(ElfinderErrorMessages.ERROR_MKDIR, name, e) }
 
-    def mkfile(self, target, name, mimes=[], debug=False):
+    def _mkfile(self, target, name):
         """
-        **Command**: Create a new, empty file.
+        **Command**: Create a new, empty file. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         try:
             volume = self._volume(target)
@@ -420,32 +436,38 @@ class ElfinderConnector:
         try:
             return { 'added' : [volume.mkfile(target, name)] }
         except NamedError as e:
-            return { 'error' : self.error(ElfinderErrorMessages.ERROR_MKFILE, name, e, e.name) }
+            return { 'error' : self.error(e, e.name, ElfinderErrorMessages.ERROR_MKFILE, name ) }
         except Exception as e:
             return { 'error': self.error(ElfinderErrorMessages.ERROR_MKFILE, name, e) }
 
-    def rename(self, target, name, mimes=[], debug=False):
+    def _rename(self, target, name):
         """
-        **Command**: Rename a file.
+        **Command**: Rename a file. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         try:
             volume = self._volume(target)
-            rm = volume.file(target)
-        except (VolumeNotFoundError, FileNotFoundError):
+        except (VolumeNotFoundError):
             return { 'error' : self.error(ElfinderErrorMessages.ERROR_RENAME, '#%s' % target, ElfinderErrorMessages.ERROR_FILE_NOT_FOUND) }
 
-        rm['realpath'] = volume.realpath(target)
         try:
-            return { 'added' : [volume.rename(target, name)], 'removed' : [rm] }
+            return { 'added' : [volume.rename(target, name)], 'removed' : volume.removed() }
         except NamedError as e:
-            return { 'error' : self.error(ElfinderErrorMessages.ERROR_RENAME, rm['name'], e, e.name) }
-        except Exception as e: 
-            return { 'error' : self.error(ElfinderErrorMessages.ERROR_RENAME, rm['name'], e) }
+            return { 'error' : self.error(e, e.name, ElfinderErrorMessages.ERROR_RENAME) }
+        except FileNotFoundError:
+            return { 'error' : self.error(ElfinderErrorMessages.ERROR_RENAME, '#%s' % target, ElfinderErrorMessages.ERROR_FILE_NOT_FOUND) }
+        except Exception as e:
+            return { 'error' : self.error(ElfinderErrorMessages.ERROR_RENAME, e) }
 
-    def duplicate(self, targets, suffix='copy', debug=False):
+    def _duplicate(self, targets, suffix='copy'):
         """
         **Command**: Duplicate a file. Create a copy with "{suffix} %d" suffix,
-        where "%d" is an integer and ``suffix`` an argument that defaults to `'copy`'.. 
+        where "%d" is an integer and ``suffix`` an argument that defaults to `'copy`'.
+        
+        This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used. 
         """
         result = { 'added' : [] }
         
@@ -464,9 +486,11 @@ class ElfinderConnector:
         
         return result
     
-    def rm(self, targets, debug=False):
+    def _rm(self, targets):
         """
-        **Command**: Remove directories or files.
+        **Command**: Remove directories or files. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         result  = {'removed' : []}
 
@@ -486,10 +510,16 @@ class ElfinderConnector:
 
         return result
     
-    def upload(self, target, FILES, html=False, mimes=[], debug=False):
+    def _upload(self, target, FILES, html=False):
         """
-        **Command**: Save uploaded files.
+        **Command**: Save uploaded files. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
+        
+        if isinstance(html, basestring):
+            html = int(html)
+        
         header = { 'Content-Type' : 'text/html; charset=utf-8' } if html else {}
         result = { 'added' : [], 'header' : header }
 
@@ -516,10 +546,18 @@ class ElfinderConnector:
 
         return result
 
-    def paste(self, targets, dst, cut=False, mimes=[], debug=False):
+    def _paste(self, targets, dst, cut=False):
         """
         **Command**: Copy/move ``targets`` files into a new destination ``dst``.
+        
+        This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
+        
+        if isinstance(cut, basestring):
+            cut = int(cut)
+
         error = ElfinderErrorMessages.ERROR_MOVE if cut else ElfinderErrorMessages.ERROR_COPY
         result = { 'added' : [], 'removed' : [] }
         
@@ -534,7 +572,7 @@ class ElfinderConnector:
             except VolumeNotFoundError:
                 result['warning'] = self.error(error, u'#%s' % target, ElfinderErrorMessages.ERROR_FILE_NOT_FOUND)
                 continue
-            
+
             try:
                 result['added'].append(dstVolume.paste(srcVolume, target, dst, cut))
             except NamedError as e:
@@ -544,9 +582,11 @@ class ElfinderConnector:
 
         return result
 
-    def get(self, target, debug=False):
+    def _get(self, target):
         """
-        **Command**: Return file contents
+        **Command**: Return file contents. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """        
         try:
             volume = self._volume(target)
@@ -555,7 +595,7 @@ class ElfinderConnector:
             return {'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, u'#%s' % target, ElfinderErrorMessages.ERROR_FILE_NOT_FOUND)}
         
         try:
-            content = volume.getContents(target)
+            content = volume.get_contents(target)
         except Exception as e:
             return {'error' : self.error(ElfinderErrorMessages.ERROR_OPEN, volume.path(target), e)}
         
@@ -569,9 +609,11 @@ class ElfinderConnector:
         
         return {'content' : content }
 
-    def put(self, target, content, mimes=[], debug=False):
+    def _put(self, target, content):
         """
-       **Command**:  Save ``content`` into a text file
+       **Command**:  Save ``content`` into a text file. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         try:
             volume = self._volume(target)
@@ -580,15 +622,17 @@ class ElfinderConnector:
             return {'error' : self.error(ElfinderErrorMessages.ERROR_SAVE, u'#%s' % target, ElfinderErrorMessages.ERROR_FILE_NOT_FOUND)}
         
         try:
-            return {'changed' : [volume.putContents(target, content)]} 
+            return {'changed' : [volume.put_contents(target, content)]} 
         except Exception as e:
             return {'error' : self.error(ElfinderErrorMessages.ERROR_SAVE, volume.path(target), e)}
 
-    def extract(self, target, mimes=[], debug=False):
+    def _extract(self, target):
         """
-        Extract files from archive
+        **Command**:  Extract files from archive. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
-        #TODO: Mimes are currently not being used
+
         try:
             volume = self._volume(target)
             volume.file(target)
@@ -600,12 +644,16 @@ class ElfinderConnector:
         except Exception as e:
             return {'error' : self.error(ElfinderErrorMessages.ERROR_EXTRACT, volume.path(target), e)}
 
-    def archive(self, targets, type_, mimes=[], debug=False):
+    def _archive(self, targets, type_):
         """
         **Command**: Create a new archive file containing all files in
-        ``targets`` param.
+        ``targets`` param. 
+        
+        This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
-        #TODO: Mimes are currently not being used
+
         try:
             volume = self._volume(targets[0])
         except:
@@ -616,20 +664,28 @@ class ElfinderConnector:
         except Exception as e:
             return {'error' : self.error(ElfinderErrorMessages.ERROR_ARCHIVE, e)}
 
-    def search(self, q, mimes=[], debug=False):
+    def _search(self, q):
         """
-        **Command**: Search files for ``q``.
+        **Command**: Search files for ``q``. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         q = q.strip()
         result = []
         for volume in self._volumes.values():
-            result += volume.search(q, mimes)
+            result += volume.search(q)
         return {'files' : result}
 
-    def info(self, targets, options=False, debug=False):
+    def _info(self, targets, options=False):
         """
-        **Command**: Return file info (used by client "places" ui)
+        **Command**: Return file info (used by client "places" ui). This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
+        
+        if isinstance(options, basestring):
+            options = int(options)
+        
         files = []
         for hash_ in targets:
             try:
@@ -645,18 +701,22 @@ class ElfinderConnector:
 
         return {'files' : files}
 
-    def dim(self, target, debug=False):
+    def _dim(self, target):
         """
-        **Command**: Return image dimmensions
+        **Command**: Return image dimensions. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """        
         try:
             return { 'dim' : self._volume(target).dimensions(target) }
         except (VolumeNotFoundError, FileNotFoundError, NotAnImageError):
             return {}
 
-    def resize(self, target, width, height, mode=None, x='0', y='0', degree='0', debug=False):
+    def _resize(self, target, width, height, mode=None, x='0', y='0', degree='0'):
         """
-        **Command**: Resize ``target`` image
+        **Command**: Resize ``target`` image. This method should not be invoked 
+        directly, the :meth:`elfinder.connector.ElfinderConnector.execute`
+        method must be used.
         """
         width, height, x, y, degree = int(width), int(height), int(x), int(y), int(degree)
         bg = ''
@@ -681,12 +741,3 @@ class ElfinderConnector:
                 if hash_.find(id_) == 0:
                     return v
         raise VolumeNotFoundError()
-
-    def _filter(self, files):
-        """
-        Remove hidden files and files with required mime types from the provided file list
-        """
-        for file_ in files:
-            if ('hidden' in file_ and file_['hidden']) or not self._default.mimeAccepted(file_['mime']):
-                files.remove(file_)
-        return files
\ No newline at end of file
